#include <stdio.h>
#include <stdlib.h>
#include "ranking_utils.h"

typedef struct {
    int index;
    double value;
} IndexedValue;

// descending order sort for indexed stationary distribution values
static int cmp_desc_struct(const void *a, const void *b) {
    const IndexedValue *ia = (const IndexedValue *)a;
    const IndexedValue *ib = (const IndexedValue *)b;
    if (ia->value < ib->value) return 1;
    if (ia->value > ib->value) return -1;
    return 0;
}

void print_top_k(const char *name, double *pi, int n, int k) {

    IndexedValue *ivals = malloc(n * sizeof(IndexedValue));
    if (!ivals) {
        fprintf(stderr, "malloc failed\n");
        return;
    }

    for (int i = 0; i < n; i++) {
        ivals[i].index = i;
        ivals[i].value = pi[i];
    }

    qsort(ivals, n, sizeof(IndexedValue), cmp_desc_struct);

    printf("Top %d entries of %s:\n", k, name);
    for (int i = 0; i < k && i < n; i++) {
        printf("  %2d: node %d -> %.8f\n", i + 1, ivals[i].index, ivals[i].value);
    }

    free(ivals);
}
